package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.payment.appstore.InApp;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.EquityVerificationDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionPayTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.payment.SubscriptionTypeEnum;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.payment.appstore.AppStoreReceiptInfo;
import cn.kdan.cloud.pdf.office.api.payment.constant.AppStoreAPIConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreOrderSucceedDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.payment.client.AppStoreClient;
import cn.kdan.cloud.pdf.office.payment.entity.Order;
import cn.kdan.cloud.pdf.office.payment.entity.RestorePurchaseLogs;
import cn.kdan.cloud.pdf.office.payment.entity.Subscription;
import cn.kdan.cloud.pdf.office.payment.properties.AppStoreProperties;
import cn.kdan.cloud.pdf.office.payment.service.AppStoreService;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.RestorePurchaseLogsService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023/6/29
 *
 * AppStoreServiceImpl
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class AppStoreServiceImpl implements AppStoreService {

    private final AppStoreClient appStoreClient;

    private final OrderService orderService;

    private final SubscriptionsService subscriptionsService;

    private final WebhookServiceImpl webhookService;

    private final ProductApi productApi;

    private final UserApi userApi;

    private final AppStoreProperties appStoreProperties;

    private final UserSubscriptionInfoApi userSubscriptionInfoApi;

    private final RestorePurchaseLogsService restorePurchaseLogsService;

    @Transactional(noRollbackFor = BackendRuntimeException.class)
    @Override
    public UserInfoVO appStoreOrderSucceed(AppStoreOrderSucceedDTO appStoreOrderSucceed) {
        log.info("AppStore 支付成功信息订单id：{}，产品信息：{},票据信息：{},\n restore：{}，",appStoreOrderSucceed.getOrderId(),appStoreOrderSucceed.getApplePayProductId(),appStoreOrderSucceed.getReceipt(),appStoreOrderSucceed.getRestore());
        // 恢复购买
        if (1 == appStoreOrderSucceed.getRestore()){
            return this.appStoreRestorePurchase(appStoreOrderSucceed);
        }

        try {
            OrdersVO orderVO = orderService.getOrderById(appStoreOrderSucceed.getOrderId());
            if (ObjectUtils.isEmpty(orderVO)) {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ORDER_DOES_NOT_EXIST);
            }
            if (!orderVO.getStatus().equals(OrderConstant.CREATED)){
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_THE_ORDER_STATUS_IS_ABNORMAL);
            }
            // 插入第三方订单id和票据信息
            Order orderChangeResult = new Order();
            orderChangeResult.setId(appStoreOrderSucceed.getOrderId());
            orderChangeResult.setResult(appStoreOrderSucceed.getReceipt());
            orderService.updateById(orderChangeResult);
            // 校验是否是同一产品id  r_product_pay表
            appStoreOrderSucceed.setRetry(true);
            List<InApp> inAppFilters = appStoreReceiptVerify(appStoreOrderSucceed);
            if (CollectionUtils.isEmpty(inAppFilters)) {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TICKET_VALIDATION_FAILED);
            }

            log.info("当前订单数据：{}",orderVO);
            InApp inApp = inAppFilters.get(0);

            if (StringUtils.isNotEmpty(inApp.getCancellationReason())) {
                // order交易失败
                log.info("order 取消交易，订单id，{}，取消原因：{},\n 值1表示客户由于应用程序内的实际或感知问题而取消了交易。值0表示交易因其他原因被取消；例如，如果客户意外购买",appStoreOrderSucceed.getOrderId(),inApp.getCancellationReason());
//                orderService.closeOrder(appStoreOrderSucceed.getOrderId());
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_USER_TRANSACTION_CANCELED_OR_FAILED);
            }

            String transactionId = inApp.getTransactionId();
            if (ObjectUtils.isNotEmpty(orderService.getOrderByThirdOrderNo(transactionId))) {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_TRANSACTION_ID_REPEAT);
            }
            String originalTransactionId = inApp.getOriginalTransactionId();

            Order orderChangeThirdTradeNo = new Order();
            orderChangeThirdTradeNo.setThirdTradeNo(originalTransactionId);
            orderChangeThirdTradeNo.setId(appStoreOrderSucceed.getOrderId());
            orderService.updateById(orderChangeThirdTradeNo);

            BigDecimal price = inApp.getTransactionId().equals(inApp.getOriginalTransactionId()) ? appStoreProperties.getIsDiscountPrice(): appStoreProperties.getNotDiscountPrice();
            // 添加订阅信息
            webhookService.handlingUserSubscriptionActivation(price,originalTransactionId,transactionId,orderVO,null);
            // 返回订阅信息
            UserInfoVO result = userApi.getInfoById(orderVO.getUserId()).getResult();
            result.setDigestPassword(null);
            return result;
        } catch (BackendRuntimeException e) {
            orderService.closeOrder(appStoreOrderSucceed.getOrderId());
            throw e;
        }
    }

    @Transactional
    @Override
    public UserInfoVO appStoreEquityVerification(EquityVerificationDTO equityVerificationDTO) {
        log.info("appStore 权益校验,用户id：{}",equityVerificationDTO.getUserId());
        AppStoreOrderSucceedDTO appStoreOrderSucceedDTO = new AppStoreOrderSucceedDTO();
        appStoreOrderSucceedDTO.setReceipt(equityVerificationDTO.getReceipt());
        appStoreOrderSucceedDTO.setApplePayProductId(equityVerificationDTO.getApplePayProductId());
        List<InApp> inApps = this.appStoreReceiptVerify(appStoreOrderSucceedDTO);
        ProductVO productVO = productApi.getProductByThirdPayId(equityVerificationDTO.getApplePayProductId()).getResult();
        UserSubscriptionInfo userSubscriptionInfo =
                userSubscriptionInfoApi.getByUserIdAndProductId(equityVerificationDTO.getUserId(), productVO.getId().toString()).getResult();

        // 当前订阅是否处于订阅中
        boolean isSubscriptionInProgress = userSubscriptionInfo.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());

        if (CollectionUtils.isEmpty(inApps)) {
            log.info("当前票据信息已无权益");
            // 校验用户当前权益状态是否无权益
            // 应当包含用户支付方式为 apple pay的所有订阅信息都应该过期
            List<SubscriptionsVO> subscriptionsVOS =
                    subscriptionsService.getSubscriptionsByPayment(equityVerificationDTO.getUserId(),PaymentMethodEnum.APPLE_PAY.getValue());
            // 异常数据
            List<SubscriptionsVO> subscriptionFilters = subscriptionsVOS.stream()
                    .filter(subscriptionsVO -> subscriptionsVO.getStatus().equals(SubscriptionConstant.actived))
                    .collect(Collectors.toList());
            // 更正异常数据
            if (CollectionUtils.isNotEmpty(subscriptionFilters)) {
                log.info("当前用户：{}，需要更正得异常订阅数据：{}",equityVerificationDTO.getUserId(),subscriptionFilters);
                subscriptionFilters.forEach(subscriptionsVO ->
                        subscriptionsService.updateSubscriptionStatus(subscriptionsVO.getId(),SubscriptionConstant.expired));
            }

            SubscriptionsVO subscriptionNew =
                    subscriptionsService.getSubscriptionsByProductId(equityVerificationDTO.getUserId(), productVO.getId().toString());
            // 查询当前用户当前产品id是否还有其他支付方式产生的有效订阅信息
            // 有：整个订阅状态应该处于订阅中，无：应处于订阅过期
            // 更正得状态值
            PDFOfficeUserSubscriptionStatusEnum userSubscriptionStatusEnum = null;
            // 时间上是否有效
            boolean isSubscriptionValid = subscriptionNew.getEndDate().compareTo(new Date()) > 0;
            log.info("isSubscriptionValid:{}，isSubscriptionInProgress：{}",isSubscriptionValid,isSubscriptionInProgress);
            if (isSubscriptionValid && !isSubscriptionInProgress) {
                userSubscriptionStatusEnum = PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS;
            } else if (!isSubscriptionValid && isSubscriptionInProgress) {
                userSubscriptionStatusEnum = PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_EXPIRED;
            }
            log.info("userSubscriptionStatusEnum:{}",userSubscriptionStatusEnum);
            if (userSubscriptionStatusEnum != null) {
                UserSubscriptionInfo userSubscriptionUpdate = new UserSubscriptionInfo();
                userSubscriptionUpdate.setId(userSubscriptionInfo.getId());
                userSubscriptionUpdate.setStatus(userSubscriptionStatusEnum.value());
                userSubscriptionInfoApi.update(userSubscriptionUpdate);
            }
        }else {
            InApp inApp = inApps.get(0);
            // auto_renew_status 校验自动续期订阅的续订状态。
            // 1 订阅将在当前订阅期结束时续订。
            // 0 客户已关闭订阅的自动续订。

            // 用户应该处于正常状态，校验用户状态是否异常
            log.info("当前用户：{}，应处于订阅中，实际状态：{}",equityVerificationDTO.getUserId(),isSubscriptionInProgress);
            if (!isSubscriptionInProgress){
                long expiresDateMsL = Long.parseLong(inApp.getExpiresDateMs());
                // 从时间戳创建Instant对象
                Instant instant = Instant.ofEpochMilli(expiresDateMsL);
                LocalDateTime endTime = LocalDateTime.ofInstant(instant, ZoneId.of("Etc/GMT+0"));
                UserSubscriptionInfo userSubscriptionUpdate = new UserSubscriptionInfo();
                userSubscriptionUpdate.setId(userSubscriptionInfo.getId());
                userSubscriptionUpdate.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
                if (userSubscriptionInfo.getEndDate().compareTo(MyDateUtils.localDateTimeToDate(endTime)) < 0) {
                    // 修正结束时间和状态
                    userSubscriptionUpdate.setEndDate(MyDateUtils.localDateTimeToDate(endTime));
                }
                userSubscriptionInfoApi.update(userSubscriptionUpdate);
            }
        }
        UserInfoVO result = userApi.getInfoById(equityVerificationDTO.getUserId()).getResult();
        result.setDigestPassword(null);
        return result;
    }

    /**
     * 票据验证
     *
     * @param appStoreOrderSucceed appStoreOrderSucceed
     * @return List<InApp>
     */
    private List<InApp> appStoreReceiptVerify(AppStoreOrderSucceedDTO appStoreOrderSucceed) {
        // 验证票据信息
        AppStoreReceiptInfo appStoreReceiptInfo = appStoreClient.verifyReceipt(appStoreOrderSucceed);
        log.info("票据信息：{}",appStoreReceiptInfo.getReceipt().getInApp());

        List<InApp> inAppFilters = appStoreReceiptInfo.getReceipt().getInApp().stream().filter(app -> {
            if (app.getProductId().equals(appStoreOrderSucceed.getApplePayProductId())) {
                String expiresDateMs = app.getExpiresDateMs();
                String inAppOwnershipType = app.getInAppOwnershipType();
                // 校验是否是支付成功的订单
                if (!AppStoreAPIConstant.INAPPOWNERSHIPTYPE_PURCHASED.equals(inAppOwnershipType)) {
                    return false;
                }
                // 校验订阅到期时间是否在当前时间之后
                long expiresDateMsL = Long.parseLong(expiresDateMs);
                // 从时间戳创建Instant对象
                Instant instant = Instant.ofEpochMilli(expiresDateMsL);
                LocalDateTime endTime = LocalDateTime.ofInstant(instant, ZoneId.of("Etc/GMT+0"));
                LocalDateTime now = LocalDateTime.now(ZoneId.of("Etc/GMT+0"));
                return now.isBefore(endTime);
            }
            return false;
        }).collect(Collectors.toList());
        log.info("过滤之后票据信息：{}",inAppFilters);
        return inAppFilters;
    }

    /**
     * 恢复购买
     *
     * @param appStoreOrderSucceed appStoreOrderSucceed
     * @return UserInfoVO
     */
    private UserInfoVO appStoreRestorePurchase(AppStoreOrderSucceedDTO appStoreOrderSucceed) {
        List<InApp> inAppFilters = appStoreReceiptVerify(appStoreOrderSucceed);
        if (CollectionUtils.isEmpty(inAppFilters)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TICKET_VALIDATION_FAILED);
        }
        InApp inApp = inAppFilters.get(0);
        if (StringUtils.isNotEmpty(inApp.getCancellationReason())) {
            log.info("order 取消交易，订单id，{}，取消原因：{},\n 值1表示客户由于应用程序内的实际或感知问题而取消了交易。值0表示交易因其他原因被取消；例如，如果客户意外购买",appStoreOrderSucceed.getOrderId(),inApp.getCancellationReason());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_USER_TRANSACTION_CANCELED_OR_FAILED);
        }
        String transactionId = inApp.getTransactionId();
        String originalTransactionId = inApp.getOriginalTransactionId();
        OrdersVO ordersVO = orderService.getOrderByThirdOrderNo(transactionId);
        if (ObjectUtils.isEmpty(ordersVO)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_CURRENT_ORDER_NOT_RECORDED);
        }

        // 判断是否是同一个用户
        String oldUserId = ordersVO.getUserId();
        String newUserId = appStoreOrderSucceed.getUserId();
        RestorePurchaseLogs restorePurchaseLog = restorePurchaseLogsService.getByOrderId(ordersVO.getId());
        if (ObjectUtils.isNotEmpty(restorePurchaseLog)) {
            oldUserId = restorePurchaseLog.getNewUserId();
        }
        log.info("恢复购买：oldUserId：{}，newUserId：{}",oldUserId,newUserId);
        if (org.springframework.util.ObjectUtils.nullSafeEquals(oldUserId,newUserId)){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_THE_SAME_USER);
        }
        // 权益转移和记录
        // 移除以前的订阅
        subscriptionsService.updateSubscriptionStatus(ordersVO.getSubscriptionId(), SubscriptionConstant.expired);
        // 根据用户id和产品id修改用户订阅状态 不是自动订阅
        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndProductId(oldUserId, ordersVO.getProductId()).getResult();
        UserSubscriptionInfo info = new UserSubscriptionInfo();
        info.setUserId(oldUserId);
        info.setProductId(ordersVO.getProductId());
        info.setPayType(PDFOfficeUserSubscriptionPayTypeEnum.NON_AUTO_SUBSCRIPTION.value());
        info.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_EXPIRED.value());
        info.setId(userSubscriptionInfo.getId());
        userSubscriptionInfoApi.update(info);
        // 新增新用户的订阅
        SubscriptionsVO subscription = subscriptionsService.getSubscriptionBySubscriptionId(ordersVO.getSubscriptionId());

        CreateSubscriptionDTO createSubscription = new CreateSubscriptionDTO();
        createSubscription.setUserId(newUserId);
        String subscriptionId = CommonUtils.generateId();
        createSubscription.setId(subscriptionId);
        // 使用恢复购买的额订单号
        createSubscription.setOrderId(ordersVO.getId());

        createSubscription.setAppId(ordersVO.getAppId());
        createSubscription.setPayment(ordersVO.getPayment());
        createSubscription.setProductId(ordersVO.getProductId());
        createSubscription.setThirdSubscriptionId(originalTransactionId);
        createSubscription.setEmail(userApi.getById(newUserId).getResult().getEmail());
        createSubscription.setBuyerType(4);//2官网购买 3后台赠送 4appstore
        createSubscription.setSubscriptionType(SubscriptionTypeEnum.MEMBER.value());//如果是0则是试用

        createSubscription.setEndDate(MyDateUtils.dateToLocalDateTime(subscription.getEndDate()));
        createSubscription.setStartDate(MyDateUtils.dateToLocalDateTime(subscription.getStartDate()));
        subscriptionsService.createSubscription(createSubscription);

        // 记录恢复购买 restore_purchase_logs
        RestorePurchaseLogs restorePurchaseLogs = new RestorePurchaseLogs();
        restorePurchaseLogs.setOldUserId(oldUserId);
        restorePurchaseLogs.setNewUserId(newUserId);
        restorePurchaseLogs.setAppleProductId(appStoreOrderSucceed.getApplePayProductId());
        restorePurchaseLogs.setTransactionId(transactionId);
        restorePurchaseLogs.setOriginalTransactionId(originalTransactionId);
        restorePurchaseLogs.setReceipt(appStoreOrderSucceed.getReceipt());
        restorePurchaseLogs.setOrderId(ordersVO.getId());
        restorePurchaseLogs.setNewSubscriptionId(subscriptionId);
        restorePurchaseLogsService.save(restorePurchaseLogs);
        // 返回订阅信息
        UserInfoVO result = userApi.getInfoById(appStoreOrderSucceed.getUserId()).getResult();
        result.setDigestPassword(null);
        return result;
    }

}
